CREATE TABLE DATASETS_SETTINGS
(
  ID                         NUMBER(10),
  ORGANIZATION_BI_CODE       VARCHAR2(20 CHAR)  NOT NULL,
  DATASET_ID                 NUMBER(10)         NOT NULL,
  DATA_DATE_RETRIEVAL_TYPE   NUMBER(10)         NOT NULL,
  DATA_PREVIOUS_DAYS_WINDOW  NUMBER(10),
  DATA_DATE_RETRIEVE_FROM    VARCHAR2(20 CHAR),
  DATA_DATE_RETRIEVE_TO      VARCHAR2(20 CHAR)
)
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE TABLE DATASETS_JOBS
(
  ID                     NUMBER(10),
  ORGANIZATION_BI_CODE   VARCHAR2(20 CHAR)      NOT NULL,
  DATASET_ID             NUMBER(10)             NOT NULL,
  DATASET_ACTOR_TYPE_ID  NUMBER(10)             NOT NULL,
  DATASET_JOB_STATUS_ID  NUMBER(10)             NOT NULL,
  START_TIMESTAMP        DATE                   NOT NULL,
  FINISH_TIMESTAMP       DATE
)
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE TABLE DATASETS_JOBS_STATUSES
(
  ID    NUMBER(10),
  NAME  VARCHAR2(200 CHAR)                      NOT NULL
)
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE TABLE DATASETS_ACTORS_TYPES
(
  ID    NUMBER(10),
  NAME  VARCHAR2(200 CHAR)                      NOT NULL
)
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE TABLE DATASETS_DATA_DATE_RET_TYPES
(
  ID    NUMBER(10),
  NAME  VARCHAR2(200 CHAR)                      NOT NULL
)
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE TABLE DATASETS
(
  ID    NUMBER(10),
  NAME  VARCHAR2(200 CHAR)                      NOT NULL
)
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


ALTER TABLE DATASETS_SETTINGS ADD (
  CONSTRAINT DATASETS_SETTINGS_PK
 PRIMARY KEY
 (ID));

ALTER TABLE DATASETS_JOBS ADD (
  CONSTRAINT DATASETS_JOBS_PK
 PRIMARY KEY
 (ID));

ALTER TABLE DATASETS_JOBS_STATUSES ADD (
  CONSTRAINT DATASETS_JOBS_STATUSES_PK
 PRIMARY KEY
 (ID));

ALTER TABLE DATASETS_ACTORS_TYPES ADD (
  CONSTRAINT DATASETS_ACTORS_TYPES_PK
 PRIMARY KEY
 (ID));

ALTER TABLE DATASETS_DATA_DATE_RET_TYPES ADD (
  CONSTRAINT DATASETS_DATE_RETRIEVAL_TYP_PK
 PRIMARY KEY
 (ID));

ALTER TABLE DATASETS ADD (
  CONSTRAINT DATASETS_PK
 PRIMARY KEY
 (ID));

ALTER TABLE DATASETS_SETTINGS ADD (
  CONSTRAINT DATASETS_SETTINGS_R02 
 FOREIGN KEY (DATA_DATE_RETRIEVAL_TYPE) 
 REFERENCES DATASETS_DATA_DATE_RET_TYPES (ID),
  CONSTRAINT DATASETS_SETTINGS_R01 
 FOREIGN KEY (DATASET_ID) 
 REFERENCES DATASETS (ID));

ALTER TABLE DATASETS_JOBS ADD (
  CONSTRAINT DATASETS_JOBS_R03 
 FOREIGN KEY (DATASET_JOB_STATUS_ID) 
 REFERENCES DATASETS_JOBS_STATUSES (ID),
  CONSTRAINT DATASETS_JOBS_R02 
 FOREIGN KEY (DATASET_ACTOR_TYPE_ID) 
 REFERENCES DATASETS_ACTORS_TYPES (ID),
  CONSTRAINT DATASETS_JOBS_R01 
 FOREIGN KEY (DATASET_ID) 
 REFERENCES DATASETS (ID));

CREATE SEQUENCE DATASETS_JOBS_SEQ
START WITH 1
INCREMENT BY 1
MINVALUE 0
NOCACHE 
NOCYCLE 
NOORDER;

CREATE OR REPLACE TRIGGER TRG_DATASETS_JOBS_ID BEFORE INSERT ON DATASETS_JOBS FOR EACH ROW
BEGIN
   SELECT DATASETS_JOBS_SEQ.NEXTVAL INTO :NEW.ID FROM DUAL;
END;
 
SET DEFINE OFF;
Insert into DATASETS_ACTORS_TYPES (ID,NAME) values ('1','BICONN app');
Insert into DATASETS_ACTORS_TYPES (ID,NAME) values ('2','Dataset update mechanism');
REM INSERTING into DATASETS_DATA_DATE_RET_TYPES
SET DEFINE OFF;
Insert into DATASETS_DATA_DATE_RET_TYPES (ID,NAME) values ('1','Time window');
Insert into DATASETS_DATA_DATE_RET_TYPES (ID,NAME) values ('2','Explicit dates');
REM INSERTING into DATASETS_JOBS_STATUSES
SET DEFINE OFF;
Insert into DATASETS_JOBS_STATUSES (ID,NAME) values ('1','Running');
Insert into DATASETS_JOBS_STATUSES (ID,NAME) values ('2','Dataset updated');
Insert into DATASETS_JOBS_STATUSES (ID,NAME) values ('3','Dataset not updated (error)');
Insert into DATASETS_JOBS_STATUSES (ID,NAME) values ('4','Dataset locked');
REM INSERTING into DATASETS
SET DEFINE OFF;
Insert into DATASETS (ID,NAME) values ('1','Patients');
Insert into DATASETS (ID,NAME) values ('2','Warehouse');
Insert into DATASETS (ID,NAME) values ('3','Payrolls');
Insert into DATASETS (ID,NAME) values ('4','Financial');
Insert into DATASETS (ID,NAME) values ('5','Appointments');

commit; 


CREATE TABLE CODIFICATION_TYPES
(
  ID    NUMBER(10),
  NAME  VARCHAR2(200 CHAR)                      NOT NULL
)
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE TABLE CODIFICATION_VALUES
(
  ID                    VARCHAR2(100 CHAR),
  ORGANIZATION_BI_CODE  VARCHAR2(20 CHAR)       NOT NULL,
  DATASET_ID            NUMBER(10)              NOT NULL,
  CODIFICATION_TYPE_ID  NUMBER(10)              NOT NULL,
  NAME                  VARCHAR2(500 CHAR)      NOT NULL,
  INSERT_TIMESTAMP      DATE                    NOT NULL,
  UPDATE_TIMESTAMP      DATE,
  IS_DELETED            NUMBER(1)               NOT NULL,
  IS_INACTIVE           NUMBER(1)               NOT NULL
)
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


ALTER TABLE CODIFICATION_TYPES ADD (
  CONSTRAINT CODIFICATION_TYPES_PK
 PRIMARY KEY
 (ID));

ALTER TABLE CODIFICATION_VALUES ADD (
  CONSTRAINT CODIFICATION_VALUES_PK
 PRIMARY KEY
 (ID, ORGANIZATION_BI_CODE, DATASET_ID, CODIFICATION_TYPE_ID));

ALTER TABLE CODIFICATION_VALUES ADD (
  CONSTRAINT CODIFICATION_VALUES_R02 
 FOREIGN KEY (CODIFICATION_TYPE_ID) 
 REFERENCES CODIFICATION_TYPES (ID),
  CONSTRAINT CODIFICATION_VALUES_R01 
 FOREIGN KEY (DATASET_ID) 
 REFERENCES DATASETS (ID));


 
SET DEFINE OFF;
Insert into CODIFICATION_TYPES (ID,NAME) values ('0','Τμήμα / Κλινική / Αυτόνομη Μονάδα');
Insert into CODIFICATION_TYPES (ID,NAME) values ('1','Μονάδα μέτρησης');
Insert into CODIFICATION_TYPES (ID,NAME) values ('2','Έκβαση νοσηλείας');
Insert into CODIFICATION_TYPES (ID,NAME) values ('3','Κατηγορία ουσίας');
Insert into CODIFICATION_TYPES (ID,NAME) values ('4','Βαθμίδα προσωπικού');
Insert into CODIFICATION_TYPES (ID,NAME) values ('5','Κατηγορία προσωπικού');
Insert into CODIFICATION_TYPES (ID,NAME) values ('6','Κλάδος προσωπικού');
Insert into CODIFICATION_TYPES (ID,NAME) values ('7','Ιατρική ειδικότητα');
Insert into CODIFICATION_TYPES (ID,NAME) values ('8','Εργασιακή σχέση');
Insert into CODIFICATION_TYPES (ID,NAME) values ('9','Υπηρεσιακή κατάσταση');
Insert into CODIFICATION_TYPES (ID,NAME) values ('10','Κατάσταση εργαζομένου');
Insert into CODIFICATION_TYPES (ID,NAME) values ('11','Είδος θέσης');
Insert into CODIFICATION_TYPES (ID,NAME) values ('12','Αιτία κάλυψης προσωποπαγούς θέσης');
Insert into CODIFICATION_TYPES (ID,NAME) values ('13','Τύπος επιδόματος');
Insert into CODIFICATION_TYPES (ID,NAME) values ('14','Οικογενειακή κατάσταση');

commit; 


create or replace function    insert_datasets_jobs(in_organization_bi_code in datasets_jobs.organization_bi_code%type,
                                                in_dataset_id           in datasets_jobs.dataset_id%type,
                                                out_dataset_job_id      out datasets_jobs.id%type,
                                                out_message             out varchar2)
  return boolean is
begin
  begin
    insert into datasets_jobs
      (organization_bi_code,
       dataset_id,
       dataset_actor_type_id,
       dataset_job_status_id,
       start_timestamp,
       finish_timestamp)
    values
      (in_organization_bi_code, in_dataset_id, 2, 1, sysdate, null)
    returning id into out_dataset_job_id;
  exception
    when others then
      out_message := 'insert_datasets_jobs.others ' || sqlerrm;
      return false;
  end;
  return true;
end insert_datasets_jobs;